var path = require( "path" );

var logger = require('./logger.js');
var util = require( './util.js' );

var platform = function () {

    var ext = util.isWin ? ".exe" : "";

    return {
        ext : ext,
        loadersDirPath : process.env.LOADERS_DIR_PATH ? process.env.LOADERS_DIR_PATH : '../loaders',
        isWin : util.isWin,
        isLinux : util.isLinux
    };

} ();

var lm4FlashLoaderSpec = {
    execFile : "./lm4flash" + platform.ext,
    workingDirPath : path.resolve( platform.loadersDirPath + "/lm4flash" ),
    getLoadProgramArgs : function ( filePath, context ) {
        return [ "-S" , context.loadAddress ? context.loadAddress : 0 , filePath ];
    },
    success : function ( err, data ) {
        if( err ) {
            return false;
        } else if ( data.toString().length === 0) {
            return  false;
        } else {
            return true;
        }
    }
};

var msp430FlashSpec = function( protocal ){
    return {
        execFile: "./mspdebug" + platform.ext,
        workingDirPath: path.resolve(platform.loadersDirPath + "/mspdebug"),
        getLoadProgramArgs: function (filePath, context) {
            return [ protocal, "--force-reset", "load " + filePath ];
        },
        getFirmwareUpdateArgs : function( context ) {
          return [protocal,"--allow-fw-update", "exit"] ;
        },
        getEraseArgs : function( context ) {
            return [protocal, "--force-reset" , "erase" ] ;
        },
        eraseSuccess : function( err, data ) {

          // we need to check the data and the err message to check if a firmware update is needed
          // on linux/windows the msg is in the data, on osx it is in the err stream.
          // js seems to have issues doing numerical comparisons on osx unless they are broken up
          var indexOfAllowFwUpdateInError = err ? err.toString().indexOf('--allow-fw-update') : -1;
          var indexOfAllowFwUpdateInData = data.indexOf('--allow-fw-update');
          var firmwareUpdateNeeded = indexOfAllowFwUpdateInData > 0 || indexOfAllowFwUpdateInError > 0;

          if( firmwareUpdateNeeded ) {
              return "FIRMWARE_UPDATE_NEEDED";
          } else if( err ) {
              return "ERROR";
          }

          return "SUCCESS";
        },
        success: function (err, data) {
            if (err) {
                return false;
            } else {
                return true;
            }
        }
    };
};

if ( platform.isWin ) {
    var cc3200LoaderSpec = {
        execFile: "./cc3200loader" + platform.ext,
        workingDirPath: path.resolve(platform.loadersDirPath + "/cc3200loader"),
        getLoadProgramArgs: function (filePath, context) {
            var loadAddress = context.loadAddress ? context.loadAddress : "0";
            var runAddress = context.runAddress ? context.runAddress : "0";
            return ["-S", loadAddress , "-e", runAddress, "--input-file" , filePath ];
        },
        success: function (err, data) {
            if (err && data.toString().length !== 0) {
                return false;
            } else {
                return true;
            }
        }
    };
} else {

    var cc3200LoaderSpec = {
        execFile: "./openocd" + platform.ext,
        workingDirPath: path.resolve(platform.loadersDirPath + "/openocd/bin"),
        getLoadProgramArgs: function (filePath, context) {

          var loadRunCmd = "load_run" + " " +  filePath + " " + context.loadAddress + " " + context.runAddress;
          var argsToReturn =  [ "-s", "../share/openocd/scripts/",
                               "-f" , "cc3200.cfg" ,
                               "-c", loadRunCmd ];

                               

          return argsToReturn;

        },
        success: function (err, data) {
            if (err && data.toString().length !== 0) {
                return false;
            } else {
                return true;
            }
        }
    };

}

var c2000LoaderSpec = {
    execFile : "./serial_loader2000" + platform.ext,
    workingDirPath : path.resolve( platform.loadersDirPath + "/c2000" ),
    getLoadProgramArgs : function ( filePath, context ) {
        return [ "-v", "-p", context.comPort , "-k", "flash_kernel.txt", "-f" , filePath ];
    },
    success : function ( err, data ) {
        if( err ) {
            return false;
        } else {
            return true;
        }
    },
    diagnosticInfo : function() {
        var info = "Please try the following to resolve this issue: \n";
            info += "1) Validate that the dip switches on the device are set appropriately\n";
            info += "2) Reset the device before attempting to flash\n";
            info += "3) Validate that the correct serial port is selected\n";
        if( platform.isLinux ) {
            info += "4) Validate that the user has the correct permissions to access the serial port\n"
        }
        return function( err, data ) {
            return info;
        };
    }()
}

var makeLoader = function ( spec ) {

    var exec = require('child_process').execFile;
        if( platform.isLinux ) {
        // set the ld library path to include current directory.. needed for msp430
        process.env.LD_LIBRARY_PATH = ".";
    }

    var firmwareUpdateExec = function( filePath, context, output ) {
        output.text( "Updating Firmware... This may take some time\n" );
        output.text ( "DO NOT UNPLUG THE DEVICE WHILE THE UPDATE IS BEING PERFOMRED!\n");
        logger.info('Args -> ' + spec.getFirmwareUpdateArgs(filePath,context));
        logger.info('Working Dir ->' + spec.workingDirPath );

        exec( spec.execFile, spec.getFirmwareUpdateArgs(filePath,context), { cwd: spec.workingDirPath }, function (err, data) {
                var message = "err : " + ( err ? err.toString() : "none" ) + " data: " + data;
                logger.info( message );
                if( spec.success( err, data ) ) {
                    output.success( "Firmware Update Successful!\n" )
                    eraseProgramExec( filePath, context, output );
                } else {
                    output.error("Firmware Update Failed!\n");
                }
            });
    };

    var loadProgramExec = function ( filePath, context, output ) {

        output.text( "Flashing Device...\n" );
        logger.info('Programming File -> ' + filePath);
        logger.info('Util -> ' + spec.execFile);
        logger.info('Args -> ' + spec.getLoadProgramArgs(filePath,context));
        logger.info('Working Dir ->' + spec.workingDirPath );

        exec( spec.execFile, spec.getLoadProgramArgs(filePath,context), { cwd: spec.workingDirPath }, function (err, data) {
                var message = "err : " + ( err ? err.toString() : "none" ) + " data: " + data;
                logger.info( message );
                if( spec.success( err, data ) ) {
                    output.success( "Flash Successful!\n" )
                } else {
                    var errormsg = "Flash Failed!\n";
                    if (spec.diagnosticInfo) {
                        errormsg += spec.diagnosticInfo(err, data);
                    }
                    output.error( errormsg );
                }
            });
    };

    var eraseProgramExec = function ( filePath, context, output ) {

        logger.info('Erasing Device');
        logger.info('Util -> ' + spec.execFile);
        logger.info('Args -> ' + spec.getEraseArgs(context) );
        logger.info('Working Dir ->' + spec.workingDirPath );

        output.text( "Erasing Device...\n" );

        exec( spec.execFile, spec.getEraseArgs( context ), { cwd: spec.workingDirPath }, function ( err, data ) {

                var message = "err : "  + ( err ? err.toString() : "none" ) + " data: " + data;
                logger.info( message );

                switch( spec.eraseSuccess( err, data ) ) {
                    case "FIRMWARE_UPDATE_NEEDED" :
                        firmwareUpdateExec( filePath, context, output );
                        break;
                    case "SUCCESS":
                        output.text( "Erase Successful!\n" );
                        loadProgramExec( filePath , context , output );
                        break;
                    case "ERROR":
                    default:
                        output.error( "Erase Failed!\n" );
                }
            });
    };

    return {

        loadProgram: function ( filePath, context, callback ) {

            // execFile is async, if calling erase, we need to wait for it to be successful before flashing device
            if ( spec.getEraseArgs ) {
                eraseProgramExec(filePath, context, callback );
            } else {
                loadProgramExec( filePath, context, callback );
            }
        }
    };
};

var flashUtilLookup = function( target ) {

    if( target.indexOf('MSP430G') >= 0 && !platform.isWin )
        return 'mspdebug_rf2500';

    if( target.indexOf('MSP') >= 0 )
        return 'mspdebug';

    if( target.indexOf('LX4') >= 0 || target.indexOf('LM4') >= 0 || target.indexOf('TM4') >= 0 )
        return 'lm4flash';

    if( target.indexOf('TMS320F28') >= 0 )
        return 'serial_loader2000';

    if( target.indexOf('CC32') >= 0 )
        return 'cc3200loader';

    return null;

};

var msp430Loader = null;
var msp430LoaderRF2500 = null;
var lm4FlashLoader = null;
var cc3200Loader = null;
var c2000Loader = null;

var getLoader = function ( target ) {

    var util = flashUtilLookup( target );

    if( util === "lm4flash" ) {
      if( lm4FlashLoader === null ) {
          lm4FlashLoader = makeLoader( lm4FlashLoaderSpec );
      }
        return lm4FlashLoader;
    } else if ( util === "mspdebug") {
        if( msp430Loader === null ) {
            msp430Loader = makeLoader( msp430FlashSpec('tilib') );
        }
        return msp430Loader;
    } else if ( util === "mspdebug_rf2500") {
        if( msp430LoaderRF2500 === null ) {
            msp430LoaderRF2500 = makeLoader( msp430FlashSpec('rf2500') );
        }
        return msp430LoaderRF2500;
    } else if ( util === "cc3200loader" ) {
        if( cc3200Loader === null ) {
            cc3200Loader = makeLoader( cc3200LoaderSpec );
        }
        return cc3200Loader;
    } else if ( util === "serial_loader2000" ) {
        if( c2000Loader === null ) {
            c2000Loader = makeLoader( c2000LoaderSpec );
        }

        return c2000Loader;
    }

    throw "Unsupported Loader Utility Type!";

};

var tempFileStorage = require( './file_storage.js' );

// States;
var STATES = {
    INIT : 0,
    WAITING_FOR_DATA : 1
};

var state = function() {
    // Init state
    var currentState = STATES.INIT;
    var context = null;
    return {
        get : function () { return currentState },
        setContext : function( value ) {
            context = value;
            if( !context ) {
                currentState = STATES.INIT;
            } else {
                switch ( currentState ) {
                    case STATES.INIT:
                        currentState = STATES.WAITING_FOR_DATA;
                        break;
                    default  :
                        currentState : STATES.INIT;
                }
            }
            return currentState;
        },
       getContext : function () {
           return context;
       }
    };

}();

// command handler
// called by the debug agent to handle load program
module.exports = function ( output ) {

    var input = {

        data : function ( inputData ) {
            var newContext;
            if (state.get() === STATES.INIT) {
                newContext = inputData;
                output.waitingForBinaryData();
            } else {

                var context = state.getContext();
                tempFilePath = tempFileStorage.storeFile( context.file, inputData );
                try {
                    loader = getLoader(context.target);
                    loader.loadProgram( tempFilePath, context, output );
                }catch( e ) {
                    output.error("Error :  " + e + "\n" );
                }

                newContext = null;
            }

            state.setContext(newContext);
        }
    };

    return input;

};

